@@ -39,6 +39,7 @@
 
 #include <glib.h>
 #include <gmodule.h>
+#include <locale.h>
 
 #include "enchant.h"
 #include "enchant-provider.h"
@@ -53,6 +54,10 @@
 #define ENCHANT_USER_PATH_EXTENSION ".enchant"
 #endif
 
+#ifdef ENABLE_BINRELOC
+#include "prefix.h"
+#endif
+
 ENCHANT_PLUGIN_DECLARE("Enchant")
 
 /********************************************************************************/
@@ -128,7 +133,9 @@ enchant_get_module_dir (void)
 	if (module_dir)
 		return module_dir;
 
-#ifdef ENCHANT_GLOBAL_MODULE_DIR
+#ifdef ENABLE_BINRELOC
+	return g_strdup (BR_LIBDIR ("/enchant"));
+#elif defined(ENCHANT_GLOBAL_MODULE_DIR)
 	return g_strdup (ENCHANT_GLOBAL_MODULE_DIR);
 #else
 	return NULL;
@@ -147,7 +154,9 @@ enchant_get_conf_dir (void)
 	if (ordering_dir)
 		return ordering_dir;
 
-#ifdef ENCHANT_GLOBAL_ORDERING
+#ifdef ENABLE_BINRELOC
+	return g_strdup (BR_DATADIR ("/enchant"));
+#elif defined(ENCHANT_GLOBAL_ORDERING)
 	return g_strdup (ENCHANT_GLOBAL_ORDERING);
 #else
 	return NULL;
@@ -200,7 +209,12 @@ enchant_get_registry_value_ex (int current_user, const char * const prefix, cons
 ENCHANT_MODULE_EXPORT (char *)
 enchant_get_registry_value (const char * const prefix, const char * const key)
 {
-	return enchant_get_registry_value_ex (0, prefix, key);
+	char *val;
+	val = enchant_get_registry_value_ex(1, prefix, key);
+	if(val == NULL) {
+		val = enchant_get_registry_value_ex (0, prefix, key);
+	}
+	return val;
 }
 
 /**
@@ -1089,25 +1103,21 @@ enchant_broker_request_pwl_dict (EnchantBroker * broker, const char *const pwl)
 {
 	EnchantSession *session;
 	EnchantDict *dict = NULL;
-	char * normalized_pwl;
 
 	g_return_val_if_fail (broker, NULL);
 	g_return_val_if_fail (pwl && strlen(pwl), NULL);
 
 	enchant_broker_clear_error (broker);
 
-	normalized_pwl = enchant_normalize_dictionary_tag (pwl);
-	dict = (EnchantDict*)g_hash_table_lookup (broker->dict_map, (gpointer) normalized_pwl);
+	dict = (EnchantDict*)g_hash_table_lookup (broker->dict_map, (gpointer) pwl);
 	if (dict) {
-		g_free (normalized_pwl);
 		return dict;
 	}
 
 	session = enchant_session_new_with_pwl (NULL, pwl, "Personal WordList", TRUE);
 	if (!session) 
 		{
-			broker->error = g_strdup_printf ("Couldn't open personal wordlist '%s'", normalized_pwl);
-			g_free (normalized_pwl);
+			broker->error = g_strdup_printf ("Couldn't open personal wordlist '%s'", pwl);
 			return NULL;
 		}
 
@@ -1116,7 +1126,7 @@ enchant_broker_request_pwl_dict (EnchantBroker * broker, const char *const pwl)
 	dict = g_new0 (EnchantDict, 1);
 	dict->enchant_private_data = (void *)session;
 
-	g_hash_table_insert (broker->dict_map, (gpointer)g_strdup (normalized_pwl), dict);
+	g_hash_table_insert (broker->dict_map, (gpointer)g_strdup (pwl), dict);
 
 	return dict;
 }
@@ -1425,3 +1435,33 @@ enchant_broker_get_error (EnchantBroker * broker)
 	
 	return broker->error;
 }
+
+/* private */ 
+ENCHANT_MODULE_EXPORT(char *)
+_enchant_get_user_language(void)
+{
+	char * locale = NULL;
+	
+#if defined(G_OS_WIN32)
+	if(!locale)
+		locale = g_win32_getlocale ();
+#endif       
+	
+	if(!locale)
+		locale = g_strdup (g_getenv ("LANG"));
+	
+#if defined(HAVE_LC_MESSAGES)
+	if(!locale)
+		locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+#endif
+
+	if(!locale)
+		locale = g_strdup (setlocale (LC_ALL, NULL));
+
+	if(!locale || strcmp(locale, "C") == 0) {
+		g_free(locale);
+		locale = g_strdup("en");
+	}
+		
+	return locale;
+}
